package api

import (
	"fmt"
	"sku3dweb/bus"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

// 会员用户
func MemberUser(r *GinRouter) {

	// 查询个人会员信息(管理员, 用户)
	r.GETJWTKEY("/memberuser/info", GetMemberUserInfo, SKU3DADMIN, SKU3DUSER)

	// 查看会员用户(管理员)
	r.GETJWTKEY("/memberuser/detail/:id", GetMemberUser, SKU3DADMIN)

	// Deprecated 查看会员用户列表(管理员, 用户)
	r.GETJWTKEY("/memberuser/list", GetMemberUsers, SKU3DADMIN, SKU3DUSER)

	// Deprecated 添加会员用户(管理员, 用户)
	r.POSTJWTKEY("/memberuser/create", CreateMemberUser, SKU3DADMIN, SKU3DUSER)

	// Deprecated 修改会员用户(管理员, 用户)
	r.POSTJWTKEY("/memberuser/update", UpdateMemberUser, SKU3DADMIN, SKU3DUSER)

	// Deprecated 删除会员用户(管理员, 用户)
	r.POSTJWTKEY("/memberuser/delete/:id", DeleteMemberUser, SKU3DADMIN)

	// 购买会员
	r.POSTJWT("/memberuser/pay", Pay)

}

func Pay(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var body struct {
		OrderId primitive.ObjectID
		PayMod  int
	}
	err := c.ShouldBindJSON(&body)
	if err != nil {
		return false, err
	}

	ctx := apictx.CreateRepoCtx()

	// 2、查询订单是否存在
	var order model.Order
	option := &repo.DocSearchOptions{
		CollectName: repo.CollectionOrder,
		Query:       repo.Map{"_id": body.OrderId},
	}
	found, _ := repo.RepoSeachDoc(ctx, option, &order)
	if !found {
		return nil, NewError("不存在该会员")
	}
	if *order.Status != model.WatiPay {
		return nil, NewError("请重新创建订单")
	}

	// 3、更新订单支付模式
	order.PayMod = &body.PayMod
	_, err = repo.RepoUpdateSetDoc(ctx, repo.CollectionOrder, order.Id.Hex(), &order)
	if err != nil {
		return nil, NewError("支付方式修改失败")
	}

	var orderName string
	if order.Member.Day == nil {
		orderName = fmt.Sprintf("会员类型:%s,使用人数:%d", order.Member.Name, *order.Member.UserNum)
	} else {
		orderName = fmt.Sprintf("会员类型:%s,天数:%d,使用人数:%d", order.Member.Name, *order.Member.Day, *order.Member.UserNum)
	}

	return bus.GetQrCode(&model.OrderMsg{Id: order.Id.Hex(), Name: orderName, PayMode: &body.PayMod, Amount: order.Amount, ModName: "SKU3D", ExpireTime: order.CreateTime.Add(time.Minute * 15)})
}

func GetMemberUserInfo(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	pidStr := apictx.User.Parent
	pid, err := primitive.ObjectIDFromHex(pidStr)
	if err != nil {
		return false, err
	}

	// 1.获取免费版
	var member model.Member
	ctx := apictx.CreateRepoCtx()
	option := &repo.DocSearchOptions{
		CollectName: repo.CollectionMember,
		Query:       repo.Map{"level": model.FREE},
	}
	found, _ := repo.RepoSeachDoc(ctx, option, &member)
	if !found {
		return []interface{}{}, err
	}

	// 1.获取会员用户,免费版亦或是企业版
	var memberusers []*model.MemberUser

	option1 := &repo.DocSearchOptions{
		CollectName: repo.CollectionMemberUser,
		Query:       repo.Map{"parentId": pid},
	}
	err = repo.RepoSeachDocs(ctx, option1, &memberusers)
	if err != nil {
		return []interface{}{}, err
	}

	// 2、没有免费版则插入
	if len(memberusers) == 0 {
		var memberuser model.MemberUser
		memberuser.ParentId = &pid
		memberuser.MemberId = member.Id
		memberuser.CreateTime = time.Now()
		memberuser.UpdateTime = time.Now()
		memberuser.DeadTime = time.Now().Add(time.Hour * 24 * 15)
		idStr, err := repo.RepoAddDoc(ctx, repo.CollectionMemberUser, memberuser)
		if err != nil {
			return []interface{}{}, err
		}
		id, _ := primitive.ObjectIDFromHex(idStr)
		memberuser.Id = &id
		memberusers = []*model.MemberUser{&memberuser}
	}

	// 3.返回前端需要的状态
	var mvos []interface{}
	for _, v := range memberusers {
		// 查询订单,加一个status字段
		var mvo struct {
			*model.MemberUser
			Status *int `json:"status,omitempty"`
		}
		mvo.MemberUser = v

		// 存在则为 免费已申请 和 会员已创建
		var order model.Order
		query := repo.Map{}

		query["userId"] = pid
		// 过滤出子账号可以获取父账号的订单
		// 过滤出申请中和等待支付的
		if v.MemberId.Hex() == member.Id.Hex() {
			query["status"] = model.Applying
		} else {
			if apictx.User.Parent != "" {
				pid, err := primitive.ObjectIDFromHex(apictx.User.Parent)
				if err != nil {
					return false, err
				}
				query["userId"] = pid
			}
			query["status"] = model.WatiPay
		}
		option := &repo.DocSearchOptions{
			CollectName: repo.CollectionOrder,
			Query:       query,
		}

		// 查出申请中、正在购买
		flag, _ := repo.RepoSeachDoc(ctx, option, &order)
		if flag {
			mvo.Status = order.Status
		}

		// 判断是否为试用免费版
		// 免费 && 创建时间+15天>现在 && 过期时间-创建时间<=15天

		if mvo.MemberId.Hex() == member.Id.Hex() && mvo.Status == nil && time.Until(v.CreateTime.Add(time.Hour*24*15)) >= 0 && v.DeadTime.Sub(v.CreateTime) <= time.Hour*24*15 {
			mvo.Status = &model.Trial
		}

		mvos = append(mvos, mvo)
	}

	return mvos, nil
}

func GetMemberUser(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	idStr := c.Param("id")
	id, err := primitive.ObjectIDFromHex(idStr)
	if err != nil {
		return false, err
	}

	var memberuser model.MemberUser

	option := &repo.DocSearchOptions{
		CollectName: repo.CollectionMemberUser,
		Query:       repo.Map{"_id": id},
	}

	// 1.存在返回，不存在插入免费映射表
	ctx := apictx.CreateRepoCtx()
	found, _ := repo.RepoSeachDoc(ctx, option, &memberuser)
	if !found {
		return false, NewError("免费版不存在")
	}

	// 2.获取企业会员用户
	// 	option1 := repo.DocSearchAggreOptions{
	// 		CollectName: repo.CollectionMemberUser,
	// 		Query: []bson.M{{"$match":bson.M{"userIds":  id}}},
	// }

	// 	var mu []*model.MemberUser
	// 	_, err = repo.RepoSeachDocAggre(ctx, &option1, &mu)
	// 	if err != nil {
	// 		return false, err
	// 	}
	// 	mu = append(mu, &memberuser)

	return memberuser, nil
}

func GetMemberUsers(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	page, size, query := UtilQueryPageSize(c)

	option := &repo.PageSearchOptions{
		CollectName: repo.CollectionMemberUser,
		Query:       query,
		Page:        page,
		Size:        size,
		Sort:        bson.M{"createTime": -1},
	}

	return repo.RepoPageSearch(apictx.CreateRepoCtx(), option)
}

func CreateMemberUser(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	var memberuser model.MemberUser
	err := c.ShouldBindJSON(&memberuser)
	if err != nil {
		return false, nil
	}

	ctx := apictx.CreateRepoCtx()

	memberuser.CreateTime = time.Now()
	memberuser.UpdateTime = time.Now()

	return repo.RepoAddDoc(ctx, repo.CollectionMemberUser, &memberuser)
}

func UpdateMemberUser(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	var memberuser model.MemberUser
	err := c.ShouldBindJSON(&memberuser)
	if err != nil {
		return false, nil
	}

	memberuser.UpdateTime = time.Now()

	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionMemberUser, memberuser.Id.Hex(), &memberuser)
}

func DeleteMemberUser(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	idStr := c.Param("id")

	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionMemberUser, idStr)
}
